SystemOrganization addCategory: #LostInMorvanLPC!
SystemOrganization addCategory: 'LostInMorvanLPC-Tests'!

SystemWindow subclass: #LPCWorld
	instanceVariableNames: 'sprites'
	classVariableNames: 'GameDirectoryName'
	poolDictionaries: ''
	category: 'LostInMorvanLPC'!

LPCWorld class
	instanceVariableNames: 'uniqueInstance'!

!LPCWorld commentStamp: 'FrancoisJarriges 5/20/2016 15:24' prior: 0!
I represent a  world for a Liberated Pixel Cup (LPC) style game where sprites can move and act.

For the Responsibility part: Three sentences about my main responsibilities - what I do, what I know.

For the Collaborators Part: State my main collaborators and one line about how I interact with them. 

Public API and Key Messages

- message one   
- message two 
- (for bonus points) how to create instances.

   One simple example is simply gorgeous.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

LPCWorld class
	instanceVariableNames: 'uniqueInstance'!

!LPCWorld class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 5/31/2016 19:10'!
createGameFromImageFiles

	| thisGameWindow mapFileReferencesArray mapMorph worldMorph scrollPane |
	
	thisGameWindow := self initialize; current.
	
	mapFileReferencesArray := (GameDirectoryName, '\images\map') asFileReference allChildrenMatching: '*.jpeg'.
	mapFileReferencesArray ifEmpty: [ ^ nil ].
	
	mapMorph := AlphaImageMorph 
		withForm: (PluginBasedJPEGReadWriter 
			formFromFileNamed: mapFileReferencesArray anyOne).

	worldMorph := PasteUpMorph new
		extent: mapMorph width @ mapMorph height;
		backgroundMorph: mapMorph;
		yourself.

	scrollPane := ScrollPane new.
	scrollPane scroller addMorph: worldMorph.

	thisGameWindow
		addMorph: scrollPane
		frame: (0@0 corner: 1@1).
	
	((GameDirectoryName, '\images\sprites') asFileReference allChildrenMatching: '*.png') do: [ :eachFileReference |
	
		| eachFileName eachSprite  eachSymbol|	
			
		eachFileName := eachFileReference basename.
		eachSprite := LPCSprite fromUniversalLPCSpritePNGFile: GameDirectoryName, '\images\sprites\', eachFileName.
		eachSymbol := eachFileReference basenameWithoutExtension asCamelCase uncapitalized.
		thisGameWindow sprites at: eachSymbol put: eachSprite ].
	
	
	thisGameWindow setLabel: 'Lost In Morvan Game'.
	
	thisGameWindow openInWorld
	
	! !

!LPCWorld class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/23/2016 09:07'!
current
	
	^ uniqueInstance ifNil: [ uniqueInstance := self new ]! !

!LPCWorld class methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/31/2016 17:25'!
initialize

	super initialize.
	self reset.
	
	GameDirectoryName := 'LPC'! !

LPCWorld initialize!

!LPCWorld class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 16:12'!
reset

	uniqueInstance := nil! !

!LPCWorld methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/24/2016 10:25'!
initialize
	
	super initialize.

	sprites := Dictionary new
	! !

!LPCWorld methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/23/2016 08:48'!
sprites
	
	^ sprites! !

TestCase subclass: #LPCSpriteTest
	instanceVariableNames: 'spritePNGFile sprite'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanLPC-Tests'!

!LPCSpriteTest commentStamp: 'FrancoisJarriges 5/22/2016 16:07' prior: 0!
I represent a TestCase for LPCSprite. The test is built on a demo sprite created at setUp by invoking :

spritePNGFile := FileSystem workingDirectory / 'LPC\PNG\sprites\lostInmorvan.png'.
sprite := LPCSprite fromUniversalLPCSpritePNGFile: spritePNGFile.

This PNG file has been dowloaded from from http://
gaurav.munjal.us/Universal-LPC-Spritesheet-Character-Generator/

The test is based on :
	checking sprite state variables at initialization and after each actions  request messages
	actions request protocol check
	corresponding animations  content check (Form collections size, Form elements class)
	
Instance Variables
	sprite:			<LPCSprite>
	spritePNGFile:	<FileReference >

!

!LPCSpriteTest methodsFor: 'running' stamp: 'FrancoisJarriges 5/22/2016 16:09'!
setUp

	spritePNGFile := FileSystem workingDirectory / 'LPC\PNG\sprites\lostInmorvan.png'.
	sprite := LPCSprite fromUniversalLPCSpritePNGFile: spritePNGFile.
! !

!LPCSpriteTest methodsFor: 'running' stamp: 'FrancoisJarriges 5/18/2016 07:19'!
tearDown

		sprite := nil.! !

!LPCSpriteTest methodsFor: 'tests' stamp: 'FrancoisJarriges 5/20/2016 13:10'!
testFromUniversalLPLCSpritePNGFile

	self assert: ((sprite tileSize / 16) isInteger and: sprite tileSize > 0).
	self assert: sprite posture isNil.
	self assert: sprite action isNil.
	self assert: sprite currentAnimation isNil.
	
	sprite act: #spellcast.
	self assert: sprite posture isNil.
	self assert: sprite action isNil.
	self assert: sprite currentAnimation isNil.
	
	sprite show: #front.
	self assert: sprite posture == #front.
	self assert: sprite action isNil.
	self assert: sprite currentAnimation isNil.
	
	sprite act: #spellcast.
	self assert: sprite posture == #front.
	self assert: sprite action == #spellcast.
	self assert: sprite currentAnimation class == OrderedCollection.
	self assert: sprite currentAnimation anyOne class == Form.
	self assert: sprite currentAnimation size = 7.
	
	sprite act: #walk.
	self assert: sprite posture == #front.
	self assert: sprite action == #walk.
	self assert: sprite currentAnimation class == OrderedCollection.
	self assert: sprite currentAnimation anyOne class == Form.
	self assert: sprite currentAnimation size = 9.
		
	sprite show: #right.
	self assert: sprite posture == #right.
	self assert: sprite action == #walk.
	self assert: sprite currentAnimation class == OrderedCollection.
	self assert: sprite currentAnimation anyOne class == Form.
	self assert: sprite currentAnimation size = 9.
	
	sprite act: #hurt.
	self assert: sprite posture == #right.
	self assert: sprite action == #hurt.
	self assert: sprite currentAnimation class == OrderedCollection.
	self assert: sprite currentAnimation size = 0.


	! !

Object subclass: #LPCSprite
	instanceVariableNames: 'spriteName tileSize posture action animations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanLPC'!

LPCSprite class
	instanceVariableNames: 'actions postures universalLPCSpritePNGFormat'!

!LPCSprite commentStamp: 'FrancoisJarriges 5/22/2016 18:48' prior: 0!
I represent a sprite in the Liberated Pixel Cup style (LPC)

I provide an instance-creation method to build the animation sequences  for a new sprite from a PNG file (as got from http://
gaurav.munjal.us/Universal-LPC-Spritesheet-Character-Generator/ ) 

The syntax is like this :

mySprite := LPCSprite fromUniversalLPCSpritePNGFile:  'LPC\PNG\sprites\lostInmorvan.png'.

LPCSprite class provide three configuration tables :

	postures 
		all possible postures for a LPCSprite  as an OrderedCollection of symbols
		
	actions 
		all possible actions for a LPCSprite  as an OrderedCollection of symbols
		
	universalLPCSpritePNGFormat
		the map of postures implemented for each action in the source PNG file as an OrderedCollection of associations

Once created, this sprite has to receive a first posture request message before treating others posture or action requests. 
This actions protocol is like that :

	mySprite show: #front .
	mySprite act: #walk.

I provide these accessing methods  :
	mySprite spriteName .		-> returns 'lostInMorvan''
	mySprite spriteName: 'a new name'.
	mySprite posture .   			-> returns #front
	mySprite action.	 	 		-> returns #walk
	mySprite currentAnimation. 	-> returns an OrderedCollection(Form(64x64x32) ...)
	mySprite tileSize.				-> returns 64 
	mySprite animationForAction: #hurt  inPosture: #back 
								-> returns an OrderedCollection() 
	
 Internal Representation and Key Implementation Points.

    Instance Variables
	spriteName 						<ByteString>
	tileSize 							<SmallInteger>
	posture  							<Symbol>
	action  							<Symbol>
	animations 						<Dictionary>
		
    Class Instance Variables 
	initialized 						<Boolean>
	actions 							<OrderedCollection>
	postures 						<OrderedCollection>	
	universalLPCSpritePNGFormat 	<OrderedCollection>
		
    Implementation Points
	an animation is represented as an OrderedCollection of poses
	each pose is an image represented as a square Form which has the size of the sprite tile (tileSize))!

LPCSprite class
	instanceVariableNames: 'actions postures universalLPCSpritePNGFormat'!

!LPCSprite class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 08:19'!
actions
	^ actions! !

!LPCSprite class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 5/23/2016 08:03'!
fromUniversalLPCSpritePNGFile: aString

	| aFileReference newSprite spriteSheet stripRank |
	
	(aString asPath basename endsWith: #png) ifFalse: [ ^ self ].
	
	newSprite := self initialize new.
	newSprite spriteName: aString asPath basenameWithoutExtension.
	
	aFileReference := FileSystem workingDirectory / aString.
	
	spriteSheet := aFileReference binaryReadStreamDo: [ :stream | 
		PNGReadWriter formFromStream: stream ].
	
	newSprite tileSize: spriteSheet height / universalLPCSpritePNGFormat size.
	
	stripRank := 0.
	
	0 to: spriteSheet height - 1 by: newSprite tileSize do: [ :animationOffset | 
		
		| animationStrip animation |
		
		animationStrip := spriteSheet copy: 
			(0 @ animationOffset corner: spriteSheet width @ (animationOffset + newSprite tileSize)).
		
		stripRank := stripRank + 1.
			
		animationStrip primCountBits > 0 ifTrue: [ 
			
			| action posture |
			
			action := (universalLPCSpritePNGFormat at: stripRank) key.
			posture := (universalLPCSpritePNGFormat at: stripRank) value.
			animation  := newSprite animationForAction: action
					inPosture: posture.
			
			0 to: spriteSheet width - 1 by: newSprite tileSize do: [ :poseOffset |
				
				| pose |
				
				pose := animationStrip copy:
					(poseOffset @ 0 corner: (poseOffset + newSprite tileSize) @ newSprite tileSize).
					
				pose primCountBits > 0 ifTrue: [ 
					animation add: pose ].
				].		
			].
		].
	^ newSprite 
	
	
! !

!LPCSprite class methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/22/2016 17:34'!
initialize

	super initialize.
	
	postures := OrderedCollection new 
		add: #back;
		add: #left;
		add: #front;
		add: #right;
		yourself.
	
	actions := OrderedCollection new
		add: #spellcast;
		add: #thrust;
		add: #walk;
		add: #slash;
		add: #shoot;
		add: #hurt;
		yourself.
		
	universalLPCSpritePNGFormat := OrderedCollection new
	
		add: #spellcast 	-> #back;
		add: #spellcast 	-> #left;
		add: #spellcast 	-> #front;
		add: #spellcast 	-> #right;
		
		add: #thrust 		-> #back;
		add: #thrust 		-> #left;
		add: #thrust 		-> #front;
		add: #thrust 		-> #right;
		
		add: #walk 			-> #back;
		add: #walk 			-> #left;
		add: #walk 			-> #front; 
		add: #walk 			-> #right; 
		
		add: #slash 			-> #back; 
		add: #slash 			-> #left;
		add: #slash 			-> #front;
		add: #slash 			-> #right;
		
		add: #shoot 			-> #back;
		add: #shoot 			-> #left;
		add: #shoot 			-> #front;
		add: #shoot 			-> #right; 
		
		add: #hurt 			-> #front; 
		
		yourself.


! !

LPCSprite initialize!

!LPCSprite class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/22/2016 17:48'!
numberOfSprites
	^ self allInstances size! !

!LPCSprite class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 10:13'!
postures
	^ postures! !

!LPCSprite methodsFor: 'actions' stamp: 'FrancoisJarriges 5/22/2016 11:34'!
act: anAction

	((self class actions includes: anAction) and: posture isNotNil) ifTrue: [ 
		action := anAction ]! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 12:32'!
action
	^ action! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/24/2016 10:23'!
animationForAction: anAction inPosture: aPosture

	((self class actions includes: anAction) and:
		(self class postures includes: aPosture)) ifFalse: [ ^ nil ].
		
	^ (animations at: anAction) at: aPosture

! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/24/2016 10:24'!
currentAnimation

	(posture isNil or: action isNil) ifTrue: [ ^ nil ].
		
	^ (animations at: action) at: posture

! !

!LPCSprite methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/24/2016 10:25'!
initialize

	super initialize.
	
	spriteName := 'spriteLPC', self class numberOfSprites asString.
	
	animations := Dictionary new.
	1 to: self class actions size do: [  :eachAction | 
		
		| actionKey |
		
		actionKey := self class actions at: eachAction.
		animations at: actionKey put: Dictionary new.
		
		1 to: self class postures size do: [ :eachPosture |
			(animations at: actionKey) 
				at: (self class postures at: eachPosture)
				put: OrderedCollection new
			]	
		]
	! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 12:32'!
posture
	^ posture! !

!LPCSprite methodsFor: 'actions' stamp: 'FrancoisJarriges 5/22/2016 11:37'!
show: aPosture

	(self class postures includes: aPosture) ifTrue: [ 
		posture := aPosture ]! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/22/2016 17:22'!
spriteName
	
	^ spriteName! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/22/2016 17:22'!
spriteName: aString
	
	spriteName := aString! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 11:29'!
tileSize

	^ tileSize ! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 11:37'!
tileSize: anInteger

	tileSize := anInteger! !

LPCWorld initialize!

LPCSprite initialize!
